(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('rxjs'), require('rxjs/operators')) :
    typeof define === 'function' && define.amd ? define('ngx-observe', ['exports', '@angular/core', 'rxjs', 'rxjs/operators'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['ngx-observe'] = {}, global.ng.core, global.rxjs, global.rxjs.operators));
}(this, (function (exports, core, rxjs, operators) { 'use strict';

    function assertTemplate(property, templateRef) {
        var isTemplateRefOrNull = !!(!templateRef || templateRef.createEmbeddedView);
        if (!isTemplateRefOrNull) {
            throw new Error(property + " must be a TemplateRef, but received '" + core.ɵstringify(templateRef) + "'.");
        }
    }
    var ObserveContext = /** @class */ (function () {
        function ObserveContext(value) {
            this.$implicit = value;
            this.ngxObserve = value;
        }
        return ObserveContext;
    }());
    var ErrorContext = /** @class */ (function () {
        function ErrorContext(error) {
            this.$implicit = error;
        }
        return ErrorContext;
    }());
    var NgxObserve = /** @class */ (function () {
        function NgxObserve(view, changes, nextTemplateRef) {
            this.view = view;
            this.changes = changes;
            this.unsubscribe = new rxjs.Subject();
            this.init = new rxjs.AsyncSubject();
            this.nextTemplateRef = nextTemplateRef;
        }
        Object.defineProperty(NgxObserve.prototype, "ngxObserve", {
            set: function (source) {
                var _this = this;
                if (this.source && source !== this.source) {
                    this.unsubscribe.next(undefined);
                }
                if (source && source !== this.source) {
                    if (this.beforeTemplateRef) {
                        this.view.clear();
                        this.nextViewRef = undefined;
                        this.errorViewRef = undefined;
                        this.beforeViewRef = this.view.createEmbeddedView(this.beforeTemplateRef);
                    }
                    this.init.pipe(operators.concatMap(function () {
                        if (_this.beforeTemplateRef) {
                            _this.view.clear();
                            _this.nextViewRef = undefined;
                            _this.errorViewRef = undefined;
                            _this.beforeViewRef = _this.view.createEmbeddedView(_this.beforeTemplateRef);
                        }
                        return source;
                    }), operators.takeUntil(this.unsubscribe)).subscribe(function (value) {
                        _this.view.clear();
                        _this.errorViewRef = undefined;
                        _this.beforeViewRef = undefined;
                        _this.nextViewRef = _this.view.createEmbeddedView(_this.nextTemplateRef, new ObserveContext(value));
                        _this.changes.markForCheck();
                    }, function (error) {
                        if (_this.errorTemplateRef) {
                            _this.view.clear();
                            _this.beforeViewRef = undefined;
                            _this.nextViewRef = undefined;
                            _this.errorViewRef = _this.view.createEmbeddedView(_this.errorTemplateRef, new ErrorContext(error));
                            _this.changes.markForCheck();
                        }
                    });
                }
                this.source = source;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxObserve.prototype, "ngxObserveError", {
            set: function (ref) {
                assertTemplate('ngxObserveError', ref);
                this.errorTemplateRef = ref;
                if (this.errorViewRef) {
                    this.view.clear();
                    this.errorViewRef = this.view.createEmbeddedView(this.errorTemplateRef, this.errorViewRef.context);
                }
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxObserve.prototype, "ngxObserveBefore", {
            set: function (ref) {
                assertTemplate('ngxObserveBefore', ref);
                this.beforeTemplateRef = ref;
                if (this.beforeViewRef) {
                    this.view.clear();
                    this.beforeViewRef = this.view.createEmbeddedView(this.beforeTemplateRef);
                }
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxObserve.prototype, "ngxObserveNext", {
            set: function (ref) {
                assertTemplate('ngxObserveNext', ref);
                this.nextTemplateRef = ref;
                if (this.nextViewRef) {
                    this.view.clear();
                    this.nextViewRef = this.view.createEmbeddedView(this.nextTemplateRef, this.nextViewRef.context);
                    this.changes.markForCheck();
                }
            },
            enumerable: false,
            configurable: true
        });
        NgxObserve.ngTemplateContextGuard = function (dir, ctx) {
            return true;
        };
        NgxObserve.prototype.ngOnDestroy = function () {
            this.unsubscribe.next(undefined);
        };
        NgxObserve.prototype.ngOnInit = function () {
            this.init.next(undefined);
            this.init.complete();
        };
        return NgxObserve;
    }());
    NgxObserve.decorators = [
        { type: core.Directive, args: [{
                    selector: '[ngxObserve]'
                },] }
    ];
    NgxObserve.ctorParameters = function () { return [
        { type: core.ViewContainerRef },
        { type: core.ChangeDetectorRef },
        { type: core.TemplateRef }
    ]; };
    NgxObserve.propDecorators = {
        ngxObserve: [{ type: core.Input }],
        ngxObserveError: [{ type: core.Input }],
        ngxObserveBefore: [{ type: core.Input }],
        ngxObserveNext: [{ type: core.Input }]
    };

    var NgxObserveModule = /** @class */ (function () {
        function NgxObserveModule() {
        }
        return NgxObserveModule;
    }());
    NgxObserveModule.decorators = [
        { type: core.NgModule, args: [{
                    declarations: [NgxObserve],
                    imports: [],
                    exports: [NgxObserve]
                },] }
    ];

    /*
     * Public API Surface of ngx-observe
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.ErrorContext = ErrorContext;
    exports.NgxObserve = NgxObserve;
    exports.NgxObserveModule = NgxObserveModule;
    exports.ObserveContext = ObserveContext;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-observe.umd.js.map
